module Bild where

import qualified SOEGraphics as SOE
import qualified GraphicsUtils as GU

import Farbe
import Transform

data Bild = Bild { gre :: Punkt -- oben rechts 
				  -- (unten links ist immer (0,0)
		 , zeichne :: Transform -> SOE.Graphic
		 }

instance Show Bild where
    showsPrec p b = showString 
		  $ "Bild { gre = " ++ show (gre b) 
		      ++ ", zeichne = ... }"

bild :: Bild -> IO ()
bild b0 = SOE.runGraphics $ do
    let b = fliphv b0
    let platz = 500 :: Int
    w <- SOE.openWindow "Bild" (platz, platz) 
    let (breit, hoch) = gre b
    let s = skaliere $ (fromInt platz / (breit `max` hoch))
    let g = zeichne b s
    SOE.drawInWindow w g
    k <- SOE.getKey w
    SOE.closeWindow w

----------------------------------------------------------------

leer :: Bild
leer = Bild { gre = (0,0)
	    , zeichne = \ t -> SOE.emptyGraphic
	    }

fliphv :: Bild -> Bild
fliphv b = 
    let (x,y) = gre b
    in   b { zeichne = \ t -> 
		     zeichne b ( t * schiebe (0,y) * hv  ) }


auf :: Bild -> Bild -> Bild
-- direkt aufeinander zeichnen
auf a b =
    let (abreit, ahoch) = gre a
	(bbreit, bhoch) = gre b
    in	Bild { gre = (abreit `max` bbreit, ahoch `max` bhoch)
	     , zeichne = \ t -> SOE.overGraphic (zeichne a t) 
						(zeichne b t)
	     }



aufs :: [ Bild ] -> Bild
aufs = foldr auf leer

neben :: Bild -> Bild -> Bild
neben a b =
    let (abreit, ahoch) = gre a
	(bbreit, bhoch) = gre b
	s = schiebe (abreit, 0)
    in	Bild { gre = (abreit + bbreit, ahoch `max` bhoch)
	     , zeichne = \ t -> SOE.overGraphic 
					    (zeichne a t)
					    (zeichne b (t * s))
	     }

nebens :: [ Bild ] -> Bild
nebens = foldl neben leer

-- aufgabe: implementiere: "zentriert nebeneinander"


ber :: Bild -> Bild -> Bild
ber a b =
    let (abreit, ahoch) = gre a
	(bbreit, bhoch) = gre b
	s = schiebe (0, ahoch)
    in	Bild { gre = (abreit `max` bbreit, ahoch + bhoch)
	     , zeichne = \ t -> SOE.overGraphic 
					    (zeichne a t)
					    (zeichne b (t * s))
	     }

bers :: [ Bild ] -> Bild
bers = foldl ber leer


-- fr leute ohne das achte bit
ueber = ber
uebers = bers


-- aufgabe: implementiere: "zentriert bereinander"


abstand :: Bild -> Bild
abstand b = 
    let (breit, hoch) = gre b
	d = 0.1 * (breit `max` hoch)
	s = schiebe (d, d)
    in	Bild { gre = (breit + 2 * d, hoch + 2 * d)
	     , zeichne = \ t -> zeichne b (t * s)
	     }

kasten :: Bild -> Bild
kasten b = 
    let c = abstand b
	(breit, hoch) = gre c
	k = polyline [(0,0), (breit,0), (breit,hoch), (0,hoch), (0,0)]
    in	c `auf` k

-------------------------------------------------------------------

farbe :: Farbe -> Bild -> Bild
farbe f b = b { zeichne = \ t -> 
		  SOE.withColor (bersetze f) $ zeichne b t }

--------------------------------------------------------------------

top_sep = 10.0 :: Double
sibling_sep = 10.0 :: Double

krone :: Bild -> [ Bild ] -> Bild
krone top kinder = 
    let n = length kinder
        (tx, ty) = gre top

	kxys = map gre kinder
	-- gesamtbreite der kinder
	kx = sum (map fst kxys) + fromIntegral (n-1) * sibling_sep
	-- gesamthhe der kinder
	ky = maximum (0 : map snd kxys)

	breit = max tx kx
	hoch  = ty + top_sep + ky

	top_offset = schiebe ( (breit - tx) / 2, hoch - ty )
	dxs = partial_sums [ x + sibling_sep | (x,y) <- kxys ]
	k_offsets = [ schiebe (dx, ky - y)
		    | (y, dx) <- zip (map snd kxys) dxs ]

	k_oben_mitten = [ (dx + x/2, ky)
			| (x, dx) <- zip (map fst kxys) dxs ]
	top_unten_mitte = ( breit / 2, ky + top_sep )
	verbindungen = aufs [ polyline0 [ m, top_unten_mitte ]
			    | m <- k_oben_mitten
			    ]

    in	Bild { gre = (breit, hoch)
	     , zeichne = \ t -> SOE.overGraphics $
		  [ zeichne top (t * top_offset)
		  , zeichne verbindungen t
		  ] ++
		  [ zeichne k (t * d)
		  | (k, d) <- zip kinder k_offsets
		  ] 
	     }

partial_sums :: Num a => [a] -> [a]
partial_sums = ps 0  where
    ps s [] = [s]
    ps s (x : xs) = s : ps (s + x) xs

-----------------------------------------------------------------

runde :: Punkt -> SOE.Point
runde (x, y) = (round x, round y)

extreme xs = (minimum xs, maximum xs)


polyline :: [ Punkt ] -> Bild
polyline = poly SOE.polyline

polygon  :: [ Punkt ] -> Bild
polygon  = poly SOE.polygon

poly f ps =
    let (xl, xh) = extreme (map fst ps)
	(yl, yh) = extreme (map snd ps)
    
    in	Bild { gre = (xh - xl, yh - yl)
	     , zeichne = \ t -> f	         
		          [ runde q'
			  | p @ (x, y) <- ps
			  , let q = (x - xl, y - yl)
			  , let q' = anwend t q
			  ]
	     }

polyline0 = poly0 $ \ ps -> 
	  ( -- SOE.withRGB (GU.RGB 0 200 100) $ 
	  SOE.polyline ps )

poly0 f ps =
    let (xl, xh) = extreme (map fst ps)
	(yl, yh) = extreme (map snd ps)
    
    in	Bild { gre = (xh, yh)
	     , zeichne = \ t -> f	         
		          [ runde q'
			  | p @ (x, y) <- ps
			  , let q' = anwend t p
			  ]
	     }

quadrat :: Double -> Bild
quadrat d = polygon [(0,0), (0,d), (d,d), (d,0)]

kreis :: Double -> Bild
kreis r = 
    let ul = (0,0); or = (2 * r, 2 * r)
    in  Bild { gre = or
	     , zeichne = \ t ->
	          let ul' = anwend t ul; or' = anwend t or
		  in  SOE.ellipse (runde ul') (runde or')
	     }

text :: String -> Bild
text cs = 
  let h = 1.0
  in
    Bild   { gre = (h, fromIntegral (length cs)) 
	    -- ob das gutgeht? buchstaben werden nicht mit skaliert
	    -- deswegen passen die abmessungen nie
	   , zeichne = \ t -> 
	          let p = runde $ anwend t (0, 2 * h)
		  in SOE.text p cs
	   }
