module Elliptisch

( ecke, regular
, Punkt, koordinaten, skaliere
, probiere
)

where

import Winkel
import Kleben

import SOEGraphics (Point)

---------------------------------------------------------------------------

ecke :: Integer ->  Rational -> Ecke Integer
-- Innenecke eines regulren hyperbolischen n-ecks
ecke n winkel =  Ecke { sichtbar = winkel
		  , alle = [ winkel ]
		  , info = [ n ]
		  }

regular :: Integer -> Rational -> Streckenzug Integer
-- ein regulres hyperbolisches n-Eck
regular n winkel = Streckenzug $ take (fromInteger n) $ repeat 
				  $ ecke n winkel

instance Correct Integer where
	 -- alles ist korrekt, was pat
    correct xs = True

---------------------------------------------------------------------------

probiere :: [ Streckenzug Integer ]
probiere = do
    let ps = -- [ regular 4 (1 % 3) ] -- viereck mit innenwinkeln von 120 grad
	     [ regular 5 (1 % 3) ] -- fnfeck mit innenwinkeln von 120 grad
    prob ps (head ps)

prob ps q = q : do
     p <- ps
     q' <- klebe q p
     prob ps q'

---------------------------------------------------------------------------

type Punkt = (Double, Double)
koordinaten :: Streckenzug i -> [ Punkt ]
koordinaten (Streckenzug ecken) = 
    koord 0 (0,0) [ w | ecke <- ecken, w <- alle ecke ]

-- durch Austausch der folgenden Funktion sollte man in die Lage kommen,
-- auch Parkettierungen in der hyperbolischen Ebene zu zeichnen!
-- man whlt einfach weiter oben Bausteine, deren Innenwinkelsumme
-- zu klein ist (z. b. ein Dreicke mit 3 * 45 Grad)
-- und rechnet dann einfach geradeaus weiter.

koord :: Winkel -> Punkt -> Winkels -> [ Punkt ]
-- aktuelle Richtung, aktuelle Position, Liste von Winkeln -> Punkte
koord w0 p0 [] = [ p0 ] -- Liste leer, wir kehren zum Anfangspunkt zurck
koord w0 p0 @ (x0, y0) (w : ws) =
    let alpha = 2 * pi * fromRational w0
	(dx, dy) = (cos alpha, sin alpha)
	p1 = (x0 + dx, y0 + dy)
	w1 = w0 + (w - 1%2) -- eventuell noch mod 1 reduzieren?
			    -- % ist der Konstruktor fr rationale Zahlen
    in	p0 : koord w1 p1 ws

skaliere :: Int -> [ Punkt ] -> [ Point ]
-- Point ist definiert in SOEGraphics als (Int, Int)
skaliere size ps = 
    let (xmin, xmax) = extreme [ x | (x,y) <- ps ]
	(ymin, ymax) = extreme [ y | (x,y) <- ps ]
	extreme zs = (minimum zs, maximum zs)    

	dx = xmax - xmin; dy = ymax - ymin
	d = max dx dy; f = fromInt size / d
	x0 = xmin - (d - dx) / 2; y0 = ymin - (d - dy) / 2

	-- was jetzt folgt, ist eine list comprehension    
    in  [ (round (f * (x - x0)), round (f * (y - y0))) 
	| (x, y) <- ps  -- das heit Generator
	]

