module Function_Parser 

( module Function_Parser
, parse
, final
)

where

import Function_Type
import Function_Form

import Position_Parser 
import Op

pm :: Parser Op
pm =      fmap ( const Plus  ) ( operator "+" )
   `plus` fmap ( const Minus ) ( operator "-" )

md :: Parser Op
md =      fmap ( const Mal   ) ( operator "*" )
   `plus` fmap ( const Durch ) ( operator "/" )

oder :: Parser Op
oder =    fmap ( const Oder  ) ( operator "|" )

falte :: Parser Op -> Parser Term -> Parser Term
falte sep p = do 
    x0 <- p
    sys <- many $ do s <- sep; y <- p; return (s, y)
    return $ foldl ( \ x (s, y) -> Apply_Op s x y ) x0 sys

reserved = [ "if", "then", "else", "let", "in" ]
name' = satisfy (not . (`elem` reserved)) name


term :: Parser Term
term = alternative

alternative :: Parser Term
alternative = falte oder summe

summe :: Parser Term
summe = falte pm produkt

produkt :: Parser Term
produkt = falte md atom

atom :: Parser Term
atom =    do n <- natural
	     return $ Constant n
   `plus` paren term
   `plus` bindung
   `plus` verzweigung
   `plus` do f <- name'
	     args <- many term
	     return $ Apply_Fun f args

verzweigung :: Parser Term
verzweigung = do
    keyword "if"
    c <- term
    keyword "then"
    y <- term
    keyword "else"
    n <- term
    return $ If c y n

bindung :: Parser Term
bindung = do
    keyword "let"
    nts <- brace $ sepBy (operator ";")
		 $ do n <- name'
		      params <- many name'
		      operator "="
		      t <- term	
		      return (n, params, t)
    keyword "in"
    body <- term
    return $ Let nts body

