module Hyperbol 

where

import Complex

import SOEGraphics

data Gerade = Gerade { mittel :: Double, radius2 :: Double }
	deriving Show

zeichne :: Gerade  -> Graphic
zeichne g = 
    let r  = sqrt $ radius2 g
    in	polyline [ skaliere $ (r :+ 0) * z + (mittel g :+ 0)
		 | phi <- [ 0, 0.1 .. 3.2 ]
		 , let z = cis phi
		 ]

type Punkt = Complex Double

bild = 500 :: Int

orig :: Num a => a 
orig = 2   

skaliere :: Punkt -> Point
skaliere p = 
    let x = round $ ( realPart p / orig * fromInt bild )
	y = round $ ( imagPart p / orig ) * fromInt bild
    in  (x, bild - y)


quad :: Punkt -> Double
quad z = realPart z ^ 2 + imagPart z ^ 2

-----------------------------------------------------------------------

durch :: Punkt -> Punkt -> Gerade
durch p q =
    let a = quad p - quad q
	b = 2 * realPart (p - q)
	m = a / b
    in	Gerade { mittel = m
	       , radius2 = quad $ p - (m :+ 0)
	       }

schnitt :: Gerade -> Gerade -> Punkt
schnitt g h = 
    let a = ( mittel g ^ 2 - radius2 g ) - ( mittel h ^ 2 - radius2 h )
	b = 2 * ( mittel g - mittel h )
	x = a / b
	y = sqrt $ radius2 g - (mittel g - x)^2
    in	x :+ y

winkel :: Gerade -> Gerade -> Double
winkel g h =
    let z = schnitt g h
	a = phase (z - (mittel g :+ 0))
	b = phase (z - (mittel h :+ 0))
    in  b - a

spiegelp :: Gerade -> Punkt -> Punkt
spiegelp g p =
    let m = mittel g :+ 0
	(r, phi) = polar $ p - m
	r' = radius2 g / r
	p' = mkPolar r' phi
    in	m + p'

spiegelg :: Gerade -> Gerade -> Gerade
spiegelg g h =
    let r = sqrt $ radius2 h
	z0 = (mittel h - r) :+ 0
	z1 = (mittel h + r) :+ 0
	u0 = spiegelp g z0
	u1 = spiegelp g z1
    in	durch u0 u1

dreheg :: Punkt -> Double -> Gerade -> Gerade
-- p liegt auf g. drehe g um p um winkel w, so da winkel g bild = w
dreheg p w g =
    let m = mittel g :+ 0
	(r, phi) = polar $ p - m
	v = phi - w
	dx = imagPart p / tan v
	m' = realPart p - dx
	r2' = dx ^ 2 + imagPart p ^ 2
    in	Gerade { mittel = m', radius2 = r2' }

-----------------------------------------------------------------------

eps = 0.0001

fu :: Gerade -> Punkt -> Double -> Punkt
-- finde q, so da w == winkel g (durch p q)
fu g p w =
    let m = mittel g :+ 0
	r = sqrt $ radius2 g

	it = iterate nher (pi/2)
	phi = fix $ it

	fix (x : y : rest) =
	    if abs (x-y) < eps then x else fix (y : rest)

	nher phi = 
	    let q = m + mkPolar r phi
		h = durch p q
		v = winkel g h
		diff = w - v
	    in	phi - 0.5 * diff

    in	m + mkPolar r phi


-- basis :: Gerade -> Gerade -> Double -> Gerade
-- finde gerade b, so da winkel g b = winkel b h = w
basis g h w =
    let top = winkel g h 
	x = schnitt g h
	m = dreheg x (- top / 2) g -- winkelhalbierende

	r = sqrt (radius2 m)
	mid = mittel m :+ 0

	it = iterate nher 1
	phi = fix $ it

	fix (x : y : rest) =
	    if abs (x-y) < eps then x else fix (y : rest)

	nher phi = 
	    let q = mid + mkPolar r phi
		b = dreheg q (-pi/2) m
		v = winkel g b
		diff = w - v
	    in	phi + 0.5 * diff

    	q = mid + mkPolar r phi
    in	dreheg q (-pi/2) m

	

-----------------------------------------------------------------------

dreieck :: Punkt -> Punkt -> (Punkt, Punkt)
dreieck m e1 =
    let g = durch m e1
	h = dreheg m (pi/3) g
	e2 = spiegelp h e1
	e3 = spiegelp g e2
    in	(e2, e3)


-----------------------------------------------------------------------

acht w x g = 
     sequence_ [ do let b = dreheg x (k * pi/4) g
		    drawInWindow w $ zeichne $ b
	       | k <- [0 .. 3 ]
	       ]

dreiecks w = do
     let p = 0 :+ 1; q = 1 :+ 1
     let g = durch p q
     let h = dreheg p (pi/4) g

     acht w p g
     
     let b = basis g h (pi/4)

     let x = schnitt h b
     acht w x b
     
     let y = schnitt g b
     acht w y b


dreieck_quadrat w gr = do

     let phi = gr * (pi / 180)
     print (phi, gr)

     let p = 0 :+ 1; q = 1.5 :+ 1.5
     let g = durch p q
     let m = dreheg p (phi / 2) g
     let h = dreheg p phi g
     let b = basis g h phi



     let x = schnitt g b; y = schnitt h b
     let p = fu m x (pi/4)
     let q = spiegelp b p
     let qx = durch q x; qy = durch q y
     
     let x' = spiegelp qy x; y' = spiegelp qx y
     let xy = durch x y; xy' = durch x y'
     let x'y = durch x' y; x'y' = durch x' y'

     print (winkel xy xy', winkel x'y x'y')
     print (winkel xy xy' * 180 / pi, winkel x'y x'y' * 180 / pi)


     sequence_ [ drawInWindow w $ zeichne l
	       | l <- [ g, m, h, b
		      , qx, qy
		      , xy, xy', x'y, x'y' 
		      ]
	       ]
     


main = maingr 45

maingr gr = runGraphics $ do

     w <- openWindow "Hyperbol" (bild, bild) 

     dreiecks w
     -- dreieck_quadrat w gr


     k <- getKey w
     closeWindow w
