module Matrix where

import Array

newtype Matrix a = Matrix ( Array (Int,Int) a ) deriving Show

instance Eq a => Eq (Matrix a) where
    Matrix a == Matrix b = 
           bounds a == bounds b -- erstmal die indizes vergleichen
	&& assocs a == assocs b -- dann alle eintrge

instance Eq a => Ord (Matrix a) where
    Matrix a `compare` Matrix b =
        error "kann Matrizen nicht ordnen"

matrix :: Num a => (Int, Int) -> [[a]] -> Matrix a
matrix (w, h) xss =
    let b = ((1,1),(w,h))
    in  Matrix $ array b
		       [ ((i, j), (xss !! (i-1)) !! (j-1))
		       | (i, j) <- range b
		       ] 



vektor :: Num a => [ a ] -> Matrix a
-- spaltenvektor
vektor xs = 
    let n = length xs
	b = ((1,1), (n,1))
    in  Matrix $ array b [ (i, xs !! (k - 1)) | i @ (k,1) <- range b ]
    
unvektor :: Num a => Matrix a -> [ a ]
unvektor (Matrix a) =
    let ((xl,yl),(xh,yh)) = bounds a
    in	if yl /= yh
	then error "unvektor: ist gar kein spaltenvektor"
	else [ a ! (x, yl)  | x <- [xl .. xh ] ]
	     


instance Num a => Num (Matrix a) where
    fromInt x = vektor [ fromInt x ]
    fromInteger x = vektor [ fromInteger x ]
    
    Matrix s + Matrix t = 
        let b = bounds s; c = bounds t
	in	if b /= c 
		then error "Matrix.(+) auf unpassende Matrizen"
		else Matrix $ array b [ (i, s ! i + t ! i) | i <- range b ]

    Matrix s * Matrix t = 
        let ((sxl, syl),(sxh,syh)) = bounds s
	    ((txl, tyl),(txh,tyh)) = bounds t
	    b =  ((sxl,tyl),(sxh,tyh))
	in	if syl /= txl || syh /= txh 
		then error "Matrix.(*) auf unpassende Matrizen"
		else Matrix $ array b
			   [ ( k, sum [ s ! (x,i) * t ! (i,y) 
				      | i <- [ syl .. syh ] ] )
			   | k @ (x, y) <- range b
			   ]
