module Mergesort where

-- das braucen wir auf jeden Fall

merge :: Ord a => [a] -> [a] -> [a]
merge [] ys = ys
merge xs [] = xs
merge (x : xs) (y : ys) =
    if x <= y then x : merge xs (y : ys)
	      else y : merge (x : xs) ys

-- klassisches Mergesort geht so (top-down)

mergesort :: Ord a => [a] -> [a]
mergesort [] = []
mergesort [x] = [x]
mergesort xs =
    let (hier, dort) = hlften xs
    in	merge (mergesort hier)
	      (mergesort dort)

hlften :: [a] -> ([a], [a])
hlften [] = ([], [])
hlften (x : xs) = 
    let (hier, dort) = hlften xs
    in	(x : dort, hier)

-- wir schreiben das jetzt bottom-up auf:

mischsort :: Ord a => [a] -> [a]
mischsort xs = schmelze (einzelne xs) -- (steigende xs)

einzelne :: [a] -> [[a]]
einzelne xs = [ [x] | x <- xs ]

paare :: Ord a => [[a]] -> [[a]]
paare [] = []; paare [xs] = [xs]
paare (xs : ys : rest) = merge xs ys : paare rest

schmelze :: Ord a => [[a]] -> [a]
schmelze []  = []
schmelze [xs] = xs
schmelze xss = schmelze (paare xss)


-- damit wir nmlich das Programm beschleunigen knnen:
-- wir beginnen nicht mit Einerfolgen,
-- sondern mit maximalen monotonen Teilfolgen:

-- mischsort xs = schmelze (steigende xs)

steigende :: Ord a => [a] -> [[a]]

-- zerlegt xs als Summe (++)
-- von monoton steigenden, nichtleeren Listen
-- Beispiel:
-- Mergesort> steigende [sin x | x <- [1..10]]
-- [[0.841471,0.909297],[0.14112],[-0.756802],[-0.958924,-0.279415,0.656987,0.989358],[0.412118],[-0.544021]]

-- Hausaufgabe: implementiere das. -- HIDE


steigende [] = []; steigende [x] = [[x]]
steigende (x : xs) =
    let ((b : bs) : rest) = steigende xs
    in	if x <= b then (x  :  b : bs) : rest
		  else [x] : (b : bs) : rest


