module Static_Eval where

-- Unterschiede zu   module Function_Eval 
-- (d. h. dynamische Bindung)
-- sind mit !!! markiert

import Function_Type
import Function_Form
import Env
import Meaning

import MonadZero
import MonadPlus

import Prelude hiding (mapM)
import MapM

-- !!!
data Function = Function ([String], Term, Envs Function)
     deriving Show

eval :: (MonadZero m, MonadPlus m) 
     => Envs Function -> Term -> m Integer
eval envs t = case t of

    Constant n  -> return n

    If c y n -> do x <- eval envs c
		   if 0 == x 
		      then eval envs y
		      else eval envs n

    Apply_Op o l r -> do let [mx, my] = map (eval envs) [l, r]
			 let f = meaning o
			 x <- mx; y <- my
			 f x y

    Apply_Fun f args -> case finds envs f of
		  Nothing -> error $ "Variable/Funktion " ++ f
				   ++ " nicht gebunden"
		  -- !!!
		  Just ( Function (params, body, static_envs) ) -> 
		      if length params /= length args
		      then error $ "Funktionsaufruf mit falscher Argumentzahl"
		      else let act = mkEnv [ (p, Function ([], a, envs)) 
					   | (p, a) <- zip params args ]
			   in  eval (act : static_envs) body 

    Let binds body ->
        let act = mkEnv [ (f, Function (ps, b, envs)) 
			| (f, ps, b) <- binds ]
	in  eval (act : envs) body

eval_top :: (MonadZero m, MonadPlus m) 
	 => Term -> m Integer
eval_top = eval nullEnvs


