module Umdrehen where

import Prelude hiding (reverse)

-- wir nehmen das als offensichtliche Definition
reverse :: [a] -> [a]
reverse [] = []
reverse (x : xs) = reverse xs ++ [x]

-- mit der wir die folgende (effizientere) Implementierung
-- vergleichen wollen
-- zu zeigen:  (3)  umdreh xs = reverse xs
umdreh :: [a] -> [a]
umdreh xs = stapel xs []    -- (4)


stapel :: [a] -> [a] -> [a]
-- zu zeigen:  (5)  stapel xs ys = reverse xs ++ ys
stapel [] ys = ys                        -- (6)
stapel (x : xs) ys = stapel xs (x : ys)  -- (7)


{- Beweise:

(3) folgt aus (4) und (5):

umdreh xs = stapel xs [] = reverse xs ++ [] = reverse xs

(5) folgt durch Induktion nach dem ersten Argument

stapel [] ys = ys = [] ++ ys = reverse [] ++ ys

stapel (x : xs) ys ) = stapel xs (x : ys) 
     = reverse xs ++ (x : ys) = reverse xs ++ [x] ++ ys
     =(2)= reverse (x : xs) ++ ys

-}

{- Aufgabe: zeige, da aus (1) und (2) folgt

(8)  reverse (reverse xs) = xs

benutze und beweise dazu einen geeigneten Hilfssatz

-- HIDE

der lautet  reverse (xs ++ ys) = reverse ys ++ reverse xs

reverse ([] ++ ys) = reverse ys = reverse ys ++ reverse []
reverse ((x : xs) ++ ys)
  = reverse (x : (xs ++ ys))
  = reverse (xs ++ ys) ++ [x]
  = (reverse ys ++ reverse xs) ++ [x]
  = reverse ys ++ (reverse xs ++ [x])
  = reverse ys ++ reverse (x : xs)

reverse (reverse (x : xs))
  = reverse (reverse xs ++ [x])
  = reverse [x] ++ reverse (reverse xs) 
  = reverse (x : []) ++ xs
  = reverse [] ++ [x] ++ xs
  = [] ++ [x] ++ xs = (x : []) ++ xs
  = x : ([] ++ xs) = x : xs
  
-}


