-- import SOEGraphics 

import Array
import Random
import IO hiding (bracket_, bracket)
import System (getArgs)

type Point = (Int, Int)

type Poly = [ Int ]

folge :: Int -> Poly -> Poly -> [ Poly ]
folge m diff start =
      let p : ps = iterate ( foldr ( \  (d, x) (y : ys) 
			          -> ((x + d + y) `mod` m ) : y : ys)
				  [0]  
		           . zip diff ) start
      in  p : takeWhile (/= p) ps

type Punkt = (Double, Double)

punkte :: Int -> [ Int ] -> [ Punkt ]
punkte m =
    foldr ( \ w (p @ (x,y) : ps) -> 
		   let phi = fromIntegral w * 2 * pi / fromIntegral m
		       (c, s) = (cos phi, sin phi)
		   in  (x + c, y + s) : p : ps
		   ) [(0, 0 )] 
   

extreme xs = ( minimum xs, maximum xs )

skaliere :: Int -> [ Punkt ] -> (Double, [ Point ])
skaliere b ps = 
    let (minx, maxx) = extreme (map fst ps)
	midx = (minx + maxx) / 2; diffx = maxx - minx
	(miny, maxy) = extreme (map snd ps)
	midy = (miny + maxy) / 2; diffy = maxy - miny
	diff = max diffx diffy
    in	( diff
	, map ( \ (x,y) 
	     -> ( round ( fromIntegral b * ( ( x - midx ) / diff + 0.5 ) )
		, round ( fromIntegral b * ( ( y - midy ) / diff + 0.5 ) )
			 ) ) ps
	)


inside lo hi x =
    let m = (hi + lo) / 2
	b = (hi - lo) / 2
    in	if x <= m then b / (hi - x) 
		  else b / (x - lo) 

linside lo hi x = inside (log lo) (log hi) (log x)

ansicht :: Int -> Int -> Poly -> (Double, String, [ Point ])
ansicht b m p = let
    nl = take (length p + 1) (repeat 0)
    ps = folge m p nl
    l  = length ps
    ds = map tail ps -- die Differenzen
    pp = length $ filter (== head ds) ds -- Symmetriezahl    
    ws = map head ps -- die Winkel allein
    xs = punkte m ws
    (d,  ys) = skaliere b xs
    
  in (	 linside 15 25 (fromIntegral pp) -- Symmetrie
      +  linside 500 1000 (fromIntegral l) -- Lnge
      +  linside 7 20 d  -- Breite

     ,    "pp: " ++ show pp
       ++ ", l: " ++ show l
       ++ ", d: " ++ show d
     , ys
     )


stern :: Int -> Int -> IO [ Point ]
stern b m = do
    ds <- sequence $ take 3 $ repeat $ randomRIO (0, m-1)
    let (d, msg, xs) = ansicht b m ds
    if d > 2.1
       then do print msg
	       return xs -- schn genug
       else stern b m -- nochmal probieren

cols = 5 :: Int
rows = 4 :: Int

modulus = 120 :: Int

bild :: Int -> IO [[Point]]
bild breit = sequence $ do
    let rand = breit `div` 10
    x <- [ 0 .. rows - 1 ]
    let dx = rand + x * (breit + rand)
    y <- [ 0 .. cols - 1 ]
    let dy = rand + y * (breit + rand)
    return $ do
        ps <- stern breit modulus
	return $ [ (px + dx, py + dy) | (px, py) <- ps ]

-- SOEGraphics output ----------------------------------------

{- 
    
screen = 150 :: Int

anzeig :: IO ()
anzeig =  runGraphics $ do

    w <- openWindow "Weihnachten" 
		    ( rows * screen * 12 `div` 10
		    , cols * screen * 12 `div` 10
		    )
    pss <- bild screen

    sequence_ $ do 
        ps <- pss
	return $ drawInWindow w $ polyline ps
    
    k <- getKey w

    closeWindow w

-}

-- xfig output ----------------------------------------------

file_prefix = [ "#FIG 3.2", 
	    "Landscape", "Center", "Inches", "Letter",
	    "100.00", "Single", "-2", "1200 2" 
	    ]
line_prefix = "2 1 0 1 0 7 100 0 -1 0.000 0 0 -1 0 0 2\n    "


schreib :: String -> IO ()
schreib fname = do

    h <- openFile fname WriteMode 
    pss <- bild 2200

    hPutStrLn h $ unlines $ file_prefix    
    sequence_ $ do 
      ps <- pss
      ((a,b),(c,d)) <- zip ps (tail ps ++ [ head ps ])
      return $ do
          hPutStr   h $ line_prefix 
	  hPutStrLn h $ concat [ show x ++ " " | x <- [a,b,c,d] ]

main = do
    [ fname ] <- getArgs
    schreib fname

    



