module Winkel

( Streckenzug (..), lnge
, Ecke (..)
, Winkel, Winkels
)

where

import List (intersperse)

---------------------------------------------------------------------------

-- das ist eine Folge von Innenwinkeln eines geschlossenen Polygons
		  
data Streckenzug i   = Streckenzug  [ Ecke i ]

lnge (Streckenzug xs) = length xs

instance Show i => Show (Streckenzug i) where
    showsPrec p (Streckenzug xs) = 
        foldr (.) id $ map (\ x -> shows x . showString "\n" ) $ xs


-- i ist eine Typvariable, die wr spter festlegen
-- wir werden nmlich in den Ecken (Farb-)Informationen brauchen
-- um festzustellen, ob das Zusammenlegen dort erlaubt ist


-- die Winkel sind angegeben als rationale Vielfache eines Vollwinkels.
-- um den Rand des Polygons zu durchlaufen, geht man durch "sichtbare" Winkel
-- um auch die innen liegenden Strecken zu durchlaufen (zum Malen),
-- geht man den Pfad durch "alle" Winkel

data Ecke i = Ecke { sichtbar :: Winkel 
		   , alle :: Winkels
		   , info :: [ i ]
		   }

-- beachte: in Haskell verwendet man gern "dutch layout",
-- das heit: ffnende Klammer, Trennzeichen (Kommas)
-- und schlieende Klammer stehen untereinander!


instance Show i => Show (Ecke i) where
    showsPrec p e 
        = shows (sichtbar e)
	. showString " / "
	. shows (info e)
	. showString " / "
	. showsPrec p (alle e)
	

type Winkel = Rational
type Winkels = [ Winkel ]

-- das klingt zwar nicht schn, ist aber sinnvoll:
-- das Anwenden des List-Operators (Typkonstruktors)
-- sollte man durch einen Suffix im Namen kennzeichnen
-- Beispiel:  x :: a, xs :: [a]
-- die Englnder habens leicht, die hngen ein "s" an,
-- im Deutschen ist der Plural manchmal -e, manchmal -n,
-- oder ganz ohne Endung, da gibt es keine schne Lsung.

instance Eq (Ecke i) where 
-- Eq ist eine Typklasse, mit einer Methode, die heit (==)
-- siehe die Standard Prelude, /usr/local/share/hugs/lib/Prelude.hs
	 w1 == w2  =  sichtbar w1 == sichtbar w2

instance Ord (Ecke i) where  -- siehe ebenfalls dort
	 w1 `compare` w2  =  sichtbar w1 `compare` sichtbar w2

-- das ist eine etwas gewagte Instanz (besonders "*")
-- aber damit kann man weiter unten schn kurzen Formeln schreiben
instance Show i => Num (Ecke i) where
	 -- alle Winkel aneinanderkleben
	 w1 + w2 = Ecke { sichtbar  = sichtbar w1 + sichtbar w2
			, alle = alle w1 ++ alle w2
			, info = info w1 ++ info w2
			}
	 -- letzen mit erstem verschmelzen
	 w1 * w2 = Ecke { sichtbar  = sichtbar w1 + sichtbar w2
			, alle = verbinde (alle w1) (alle w2)
			, info = info w1 ++ info w2
			}

verbinde :: Winkels -> Winkels -> Winkels
verbinde ps qs =
    let (p_vorn, [p_letzer]) = splitAt (length ps - 1) ps
	([q_erster], q_hinten) = splitAt 1 qs
    in  p_vorn ++ [ p_letzer + q_erster ] ++ q_hinten

